@@ -4,6 +4,8 @@ from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
 import logging
 
+import dateutil.tz
+
 from trytond.model import fields
 from trytond.tools import get_smtp_server
 from trytond.transaction import Transaction
@@ -11,6 +13,7 @@ from trytond.pool import Pool, PoolMeta
 
 __all__ = ['Event', 'EventAttendee']
 __metaclass__ = PoolMeta
+tzlocal = dateutil.tz.tzlocal()
 
 logger = logging.getLogger(__name__)
 
@@ -36,7 +39,7 @@ class Event:
             ], 'Schedule Agent')
 
     @staticmethod
-    def default_schedule_agent():
+    def default_organizer_schedule_agent():
         return 'SERVER'
 
     @classmethod
@@ -97,6 +100,9 @@ class Event:
                 (self.organizer_schedule_status,)
 
         if Transaction().context.get('skip_schedule_agent'):
+            if (hasattr(vevent, 'organizer')
+                    and hasattr(vevent.organizer, 'schedule_agent_param')):
+                del vevent.organizer.schedule_agent_param
             return ical
 
         if self.organizer_schedule_agent:
@@ -124,18 +130,28 @@ class Event:
                 summary = self.raise_user_error('no_subject',
                         raise_exception=False)
 
-        date = Lang.strftime(self.dtstart, lang.code, lang.date)
+        if self.timezone:
+            tzevent = dateutil.tz.gettz(self.timezone)
+        else:
+            tzevent = tzlocal
+        dtstart = self.dtstart.replace(tzinfo=tzlocal).astimezone(tzevent)
+        if self.dtend:
+            dtend = self.dtend.replace(tzinfo=tzlocal).astimezone(tzevent)
+        else:
+            dtend = None
+
+        date = Lang.strftime(dtstart, lang.code, lang.date)
         if not self.all_day:
-            date += ' ' + Lang.strftime(self.dtstart, lang.code, '%H:%M')
+            date += ' ' + Lang.strftime(dtstart, lang.code, '%H:%M')
             if self.dtend:
                 date += ' -'
                 if self.dtstart.date() != self.dtend.date():
-                    date += ' ' + Lang.strftime(self.dtend, lang.code,
+                    date += ' ' + Lang.strftime(dtend, lang.code,
                         lang.date)
-                date += ' ' + Lang.strftime(self.dtend, lang.code, '%H:%M')
+                date += ' ' + Lang.strftime(dtend, lang.code, '%H:%M')
         else:
             if self.dtend and self.dtstart.date() != self.dtend.date():
-                date += ' - ' + Lang.strftime(self.dtend, lang.code, lang.date)
+                date += ' - ' + Lang.strftime(dtend, lang.code, lang.date)
         if self.timezone:
             date += ' ' + self.timezone
 
@@ -201,11 +217,9 @@ class Event:
         msg_body.set_payload(body.encode('UTF-8'), 'UTF-8')
         inner.attach(msg_body)
 
-        attachment = MIMEBase('text', 'calendar')
-        attachment.set_payload(ical.serialize())
-        attachment.add_header('Content-Transfer-Encoding', 'quoted-printable',
-                charset='UTF-8',
-                method=ical.method.value.lower())
+        attachment = MIMEBase('text', 'calendar',
+            method=ical.method.value)
+        attachment.set_payload(ical.serialize(), 'UTF-8')
         inner.attach(attachment)
 
         msg.attach(inner)
@@ -322,11 +336,11 @@ class Event:
             return
 
         actions = iter(args)
+        events_edited = set()
         for events, values in zip(actions, actions):
-            event_edited = False
             for k in values:
                 if k != 'attendees':
-                    event_edited = True
+                    events_edited.update(events)
                     break
 
             # store old attendee info
@@ -373,7 +387,7 @@ class Event:
 
             sent_succes = []
             sent_fail = []
-            if event_edited:
+            if event in events_edited:
                 if event.status == 'cancelled':
                     ical.method.value = 'CANCEL'
                     # send cancel to old attendee
@@ -434,9 +448,11 @@ class Event:
 
                 vals = {'status': 'needs-action'}
                 vals['schedule_status'] = '1.1'  # successfully sent
-                Attendee.write(sent_succes, vals)
+                if sent_succes:
+                    Attendee.write(sent_succes, vals)
                 vals['schedule_status'] = '5.1'  # could not complete delivery
-                Attendee.write(sent_fail, vals)
+                if sent_fail:
+                    Attendee.write(sent_fail, vals)
 
     @classmethod
     def delete(cls, events):
@@ -496,6 +512,10 @@ class AttendeeMixin:
 
     @classmethod
     def attendee2values(cls, attendee):
+        # Those params don't need to be stored
+        for param in ['received_dtstamp_param', 'received_sequence_param']:
+            if hasattr(attendee, param):
+                delattr(attendee, param)
         values = super(AttendeeMixin, cls).attendee2values(attendee)
         if hasattr(attendee, 'schedule_status'):
             if attendee.schedule_status in dict(
@@ -522,6 +542,8 @@ class AttendeeMixin:
                 del attendee.schedule_status_param
 
         if Transaction().context.get('skip_schedule_agent'):
+            if hasattr(attendee, 'schedule_agent_param'):
+                del attendee.schedule_agent_param
             return attendee
 
         if self.schedule_agent:
@@ -551,7 +573,7 @@ class EventAttendee(AttendeeMixin, object):
                 'body': ('%s (%s) changed his/her participation status '
                     'to: %s\n\n'),
                 'accepted_body': '%s (%s) has accepted this invitation:\n\n',
-                'declined_body': '%s (%s) has declined this invitattion:\n\n',
+                'declined_body': '%s (%s) has declined this invitation:\n\n',
                 'no_subject': "(No Subject)",
                 'separator': ':',
                 'bullet': '    * ',
@@ -578,18 +600,28 @@ class EventAttendee(AttendeeMixin, object):
                 summary = self.raise_user_error('no_subject',
                         raise_exception=False)
 
-        date = Lang.strftime(event.dtstart, lang.code, lang.date)
+        if event.timezone:
+            tzevent = dateutil.tz.gettz(event.timezone)
+        else:
+            tzevent = tzlocal
+        dtstart = event.dtstart.replace(tzinfo=tzlocal).astimezone(tzevent)
+        if event.dtend:
+            dtend = event.dtend.replace(tzinfo=tzlocal).astimezone(tzevent)
+        else:
+            dtend = None
+
+        date = Lang.strftime(dtstart, lang.code, lang.date)
         if not event.all_day:
-            date += ' ' + Lang.strftime(event.dtstart, lang.code, '%H:%M')
+            date += ' ' + Lang.strftime(dtstart, lang.code, '%H:%M')
             if event.dtend:
                 date += ' -'
                 if event.dtstart.date() != event.dtend.date():
-                    date += ' ' + Lang.strftime(event.dtend, lang.code,
+                    date += ' ' + Lang.strftime(dtend, lang.code,
                         lang.date)
-                date += ' ' + Lang.strftime(event.dtend, lang.code, '%H:%M')
+                date += ' ' + Lang.strftime(dtend, lang.code, '%H:%M')
         else:
             if event.dtend and event.dtstart.date() != event.dtend.date():
-                date += ' - ' + Lang.strftime(event.dtend, lang.code,
+                date += ' - ' + Lang.strftime(dtend, lang.code,
                     lang.date)
         if event.timezone:
             date += ' ' + event.timezone
@@ -637,7 +669,7 @@ class EventAttendee(AttendeeMixin, object):
                         body += owner.email + bullet + '\n'
                     for attendee in event.attendees:
                         body += attendee.email + bullet + '\n'
-            elif event[field]:
+            elif getattr(event, field):
                 if field == 'summary':
                     value = summary
                 elif field == 'dtstart':
@@ -671,11 +703,9 @@ class EventAttendee(AttendeeMixin, object):
         msg_body.set_payload(body.encode('UTF-8'), 'UTF-8')
         inner.attach(msg_body)
 
-        attachment = MIMEBase('text', 'calendar')
-        attachment.set_payload(ical.serialize())
-        attachment.add_header('Content-Transfer-Encoding', 'quoted-printable',
-                charset='UTF-8',
-                method=ical.method.value.lower())
+        attachment = MIMEBase('text', 'calendar',
+            method=ical.method.value)
+        attachment.set_payload(ical.serialize(), 'UTF-8')
         inner.attach(attachment)
 
         msg.attach(inner)
@@ -754,6 +784,8 @@ class EventAttendee(AttendeeMixin, object):
 
             with Transaction().set_context(skip_schedule_agent=True):
                 ical = attendee.event.event2ical()
+                # Only the current attendee is needed
+                ical.vevent.attendee_list = [attendee.attendee2attendee()]
             if not hasattr(ical, 'method'):
                 ical.add('method')
             ical.method.value = 'REPLY'
@@ -789,6 +821,8 @@ class EventAttendee(AttendeeMixin, object):
 
             with Transaction().set_context(skip_schedule_agent=True):
                 ical = attendee.event.event2ical()
+                # Only the current attendee is needed
+                ical.vevent.attendee_list = [attendee.attendee2attendee()]
             if not hasattr(ical, 'method'):
                 ical.add('method')
             ical.method.value = 'REPLY'
@@ -828,6 +862,8 @@ class EventAttendee(AttendeeMixin, object):
 
             with Transaction().set_context(skip_schedule_agent=True):
                 ical = attendee.event.event2ical()
+                # Only the current attendee is needed
+                ical.vevent.attendee_list = [attendee.attendee2attendee()]
             if not hasattr(ical, 'method'):
                 ical.add('method')
             ical.method.value = 'REPLY'
